﻿namespace Galatea.Data.Mappers
{
    using System;
    using System.Globalization;
    using FluentAssertions;
    using Galatea.DomainModel;
    using NUnit.Framework;
    using Galatea.Data.TransportObjects;

    [TestFixture]
    public class UserDataMapperTest
    {
        [SetUp]
        public void SetUp()
        {
            this.Target = new UserDataMapper();
        }

        /// <summary>
        /// Gets or sets the object which is being tested.
        /// </summary>
        private UserDataMapper Target { get; set; }

        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void ToDataTransportRejectsNullModel()
        {
            // Calling ToDataTransport with a null domain model should throw an
            // argument null exception...
            this.Target.ToDataTransport(null);
        }

        [Test]
        public void ToDataTransportSucceeds()
        {
            // Set up a test user domain model...
            var model = new User
            {
                Culture      = new CultureInfo("en-GB"),
                EmailAddress = "test@test.com",
                LastLoginUtc = DateTimeOffset.UtcNow.AddDays(-7),
                PasswordHash = "ABCDEF",
                UserGuid     = Guid.NewGuid(),
            };

            // Construct a DTO from the test domain model...
            var result = this.Target.ToDataTransport(model);

            // Assert that the resulting DTO has been correctly formed...
            result.CultureCode.Should().Be(model.Culture.IetfLanguageTag);
            result.EmailAddress.Should().Be(model.EmailAddress);
            result.LastLoginUtc.Should().Be(model.LastLoginUtc);
            result.PasswordHash.Should().Be(model.PasswordHash);
            result.UserGuid.Should().Be(model.UserGuid);
        }

        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void FromDataTransportRejectsNullDTO()
        {
            Target.FromDataTransport(null);
        }

        [Test]
        public void FromDataTransportSucceeds()
        {
            UserDTO dto = new UserDTO
            {
                CultureCode  = "en-GB",
                EmailAddress = "nickybabe@me.com",
                LastLoginUtc = DateTimeOffset.Now,
                PasswordHash = "gkjsdfglkjnsdfg",
                UserGuid     = Guid.NewGuid()
            };

            User result = Target.FromDataTransport(dto);

            result.Culture.IetfLanguageTag.Should().Be(dto.CultureCode);
            result.EmailAddress.Should().Be(dto.EmailAddress);
            result.LastLoginUtc.Should().Be(dto.LastLoginUtc);
            result.PasswordHash.Should().Be(dto.PasswordHash);
            result.UserGuid.Should().Be(dto.UserGuid);
        }
    }
}