﻿namespace Galatea.DomainModel
{
    using System;
    using System.ComponentModel;
    using System.Diagnostics.Contracts;
    using System.Threading;
    using Galatea.Data.TransportObjects;

    /// <summary>
    /// Defines the domain model object representing a Project.
    /// </summary>
    public class Project : INotifyPropertyChanged
    {
        /// <summary>
        /// Prevents a default instance of the <see cref="T:Galatea.DomainModel.Project"/> class
        /// from being created.
        /// </summary>
        internal Project()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="T:Galatea.DomainModel.Project"/> class.
        /// </summary>
        /// <param name="owner">A domain model object representing the project owner.</param>
        /// <param name="name">The name of the project.</param>
        public Project(User owner, String name)
            : this()
        {
            Contract.Requires(owner != null);
            Contract.Requires(String.IsNullOrWhiteSpace(name) != true);

            this.Name           = name;
            this.DateCreatedUtc = DateTimeOffset.UtcNow;
        }

        /// <summary>
        /// Occurs when a property value changes.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Raises the <see cref="E:Galatea.DomainModel.Project.PropertyChanged"/> event.
        /// </summary>
        /// <param name="e">The <see cref="System.ComponentModel.PropertyChangedEventArgs"/>
        /// instance containing the event data.</param>
        protected virtual void OnPropertyChanged(PropertyChangedEventArgs e)
        {
            var handler = Interlocked.CompareExchange(ref this.PropertyChanged, null, null);

            if (handler != null)
                handler(this, e);
        }

        /// <summary>
        /// Gets or sets the project GUID.
        /// </summary>
        public Guid ProjectGuid { get; internal set; }

        /// <summary>
        /// Gets or sets the unique identifier of the user which owns the project.
        /// </summary>
        public Guid OwnerGuid { get; internal set; }

        private String _name;

        /// <summary>
        /// Gets or sets the project name.
        /// </summary>
        /// <remarks>
        /// Setting this property will raise the <see cref="E:System.ComponentModel.INotifyPropertyChanged.PropertyChanged"/>
        /// event.
        /// </remarks>
        /// <exception cref="T:System.ArgumentNullException">
        /// Thrown if the value passed to the mutator accessor is null.
        /// </exception>
        public String Name
        {
            get { return this._name; }
            internal set
            {
                if (value != this.Name)
                {
                    this._name = value;

                    this.OnPropertyChanged(new PropertyChangedEventArgs("Name"));
                }
            }
        }

        /// <summary>
        /// Gets or sets the date &amp; time on which the project was created.
        /// </summary>
        public DateTimeOffset DateCreatedUtc { get; set; }

        /// <summary>
        /// Renames the project with a new name.
        /// </summary>
        /// <param name="newName">The new name of the project.</param>
        public void Rename(String newName)
        {
            Contract.Requires(String.IsNullOrWhiteSpace(newName) != true);

            this.Name = newName;
        }
    }
}
