﻿namespace Galatea.DomainModel
{
    using System;
    using System.ComponentModel;
    using System.Diagnostics.Contracts;
    using System.Linq;
    using System.Threading;
    using Galatea.Data.TransportObjects;

    /// <summary>
    /// Defines a domain model object representing a Review.
    /// </summary>
    public class Review : INotifyPropertyChanged
    {
        /// <summary>
        /// Creates a new review domain object from the specified data transport object.
        /// </summary>
        /// <param name="data">A data transport object representing a review.</param>
        /// <returns>A new domain object representing a review.</returns>
        static internal Review FromDataTransport(ReviewDTO data)
        {
            Contract.Requires(data != null);

            return new Review
            {
                Content        = data.Content,
                DateCreatedUtc = data.DateCreatedUtc,
                Description    = data.Description,
                OwnerGuid      = data.OwnerGuid,
                Revisions      = new RevisionCollection(data.Revisions.Select(Revision.FromDataTransport)),
                Title          = data.Title
            };
        }

        /// <summary>
        /// Constructs a new data transport object representing this review.
        /// </summary>
        /// <returns>A new data transport object representing this review.</returns>
        internal ReviewDTO ToDataTransport()
        {
            return new ReviewDTO
            {
                Content        = this.Content,
                DateCreatedUtc = this.DateCreatedUtc,
                Description    = this.Description,
                OwnerGuid      = this.OwnerGuid,
                Revisions      = this.Revisions.Select(x => x.ToDataTransport()).ToArray(),
                Title          = this.Title
            };
        }

        /// <summary>
        /// Occurs when a property value changes.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Raises the <see cref="E:System.ComponentModel.INotifyPropertyChanged.PropertyChanged"/>
        /// event.
        /// </summary>
        /// <param name="e">The <see cref="System.ComponentModel.PropertyChangedEventArgs"/>
        /// instance containing the event data.</param>
        protected virtual void OnPropertyChanged(PropertyChangedEventArgs e)
        {
            var handler = Interlocked.CompareExchange(ref this.PropertyChanged, null, null);

            if (handler != null)
                handler(this, e);
        }

        /// <summary>
        /// Prevents a default instance of the <see cref="T:Galatea.DomainModel.Review"/> class
        /// from being created.
        /// </summary>
        internal Review()
        {
            this.Revisions = new RevisionCollection();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="T:Galatea.DomainModel.Review"/> class.
        /// </summary>
        /// <param name="ownerGuid">The unique identifier of the user which owns the review.</param>
        /// <param name="title">The review title.</param>
        /// <param name="description">A short desciption of the review.</param>
        /// <param name="content">The text content being reviewed.</param>
        /// <exception cref="T:System.ArgumentNullException">
        /// Thrown if any of the values passed to the <paramref name="title"/>,
        /// <paramref name="description"/> or <paramref name="content"/> parameters are null or
        /// empty.
        /// </exception>
        public Review(Guid ownerGuid, String title, String description, String content)
            : this()
        {
            Contract.Requires(String.IsNullOrWhiteSpace(title) != true);
            Contract.Requires(String.IsNullOrWhiteSpace(description) != true);
            Contract.Requires(String.IsNullOrWhiteSpace(content) != true);

            this.OwnerGuid      = ownerGuid;
            this.Title          = title;
            this.Description    = description;
            this.Content        = content;
            this.DateCreatedUtc = DateTimeOffset.UtcNow;
        }

        private String _title;

        /// <summary>
        /// Gets or sets the title of the review.
        /// </summary>
        /// <remarks>
        /// Setting this property will raise the <see cref="E:System.ComponentModel.INotifyPropertyChanged.PropertyChanged"/>
        /// event.
        /// </remarks>
        /// <exception cref="T:System.ArgumentNullException">
        /// Thrown if the value passed to the mutator accessor is null.
        /// </exception>
        public String Title
        {
            get { return this._title; }
            internal set
            {
                if (value != this.Title)
                {
                    this._title = value;

                    this.OnPropertyChanged(new PropertyChangedEventArgs("Title"));
                }
            }
        }

        private String _description;

        /// <summary>
        /// Gets or sets the review description.
        /// </summary>
        /// <remarks>
        /// Setting this property will raise the <see cref="E:System.ComponentModel.INotifyPropertyChanged.PropertyChanged"/>
        /// event.
        /// </remarks>
        /// <exception cref="T:System.ArgumentNullException">
        /// Thrown if the value passed to the mutator accessor is null.
        /// </exception>
        public String Description
        {
            get { return this._description; }
            internal set
            {
                if (value != this.Description)
                {
                    this._description = value;

                    this.OnPropertyChanged(new PropertyChangedEventArgs("Description"));
                }
            }
        }

        private String _content;

        /// <summary>
        /// Gets or sets the review content.
        /// </summary>
        /// <remarks>
        /// Setting this property will raise the <see cref="E:System.ComponentModel.INotifyPropertyChanged.PropertyChanged"/>
        /// event.
        /// </remarks>
        /// <exception cref="T:System.ArgumentNullException">
        /// Thrown if the value passed to the mutator accessor is null.
        /// </exception>
        public String Content
        {
            get { return this._content; }
            internal set
            {
                if (value != this.Content)
                {
                    this._content = value;

                    this.OnPropertyChanged(new PropertyChangedEventArgs("Content"));
                }
            }
        }

        /// <summary>
        /// Gets the unique identifier of the user which owns this review.
        /// </summary>
        public Guid OwnerGuid { get; internal set; }

        /// <summary>
        /// Gets the date &amp; time on which the review was created.
        /// </summary>
        public DateTimeOffset DateCreatedUtc { get; internal set; }

        /// <summary>
        /// Gets the review content revisions collection.
        /// </summary>
        public RevisionCollection Revisions { get; private set; }

        /// <summary>
        /// Updates the title of the review.
        /// </summary>
        /// <param name="newTitle">The new title.</param>
        public void UpdateTitle(String newTitle)
        {
            Contract.Requires(String.IsNullOrWhiteSpace(newTitle) != true);

            this.Title = newTitle;
        }
    }
}