﻿namespace Galatea.DomainModel
{
    using System;
    using System.ComponentModel;
    using System.Diagnostics.Contracts;
    using System.Threading;
    using Galatea.Data.TransportObjects;

    /// <summary>
    /// Defines the domain model object representing a revision.
    /// </summary>
    public class Revision : INotifyPropertyChanged
    {
        /// <summary>
        /// Creates a new domain object from the specified data transport obejct.
        /// </summary>
        /// <param name="data">A data transport object representing a revision.</param>
        /// <returns>A new domain object representing a revision.</returns>
        static internal Revision FromDataTransport(RevisionDTO data)
        {
            Contract.Requires(data != null);

            return new Revision
            {
                Comment          = data.Comment,
                DateSubmittedUtc = data.DateSubmittedUtc,
                ReviewerGuid     = data.ReviewerGuid,
                RevisedContent   = data.RevisedContent
            };
        }

        /// <summary>
        /// Creates a new data transport object representing this revision.
        /// </summary>
        /// <returns>A new data transport object representing this revision.</returns>
        internal RevisionDTO ToDataTransport()
        {
            return new RevisionDTO
            {
                Comment          = this.Comment,
                DateSubmittedUtc = this.DateSubmittedUtc,
                ReviewerGuid     = this.ReviewerGuid,
                RevisedContent   = this.RevisedContent
            };
        }

        /// <summary>
        /// Occurs when a property value changes.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Raises the <see cref="E:System.ComponentModel.INotifyPropertyChanged.PropertyChanged"/>
        /// event.
        /// </summary>
        /// <param name="e">The <see cref="System.ComponentModel.PropertyChangedEventArgs"/>
        /// instance containing the event data.</param>
        protected virtual void OnPropertyChanged(PropertyChangedEventArgs e)
        {
            var handler = Interlocked.CompareExchange(ref this.PropertyChanged, null, null);

            if (handler != null)
                handler(this, e);
        }

        /// <summary>
        /// Prevents a default instance of the <see cref="T:Galatea.DomainModel.Revision"/> class
        /// from being created.
        /// </summary>
        internal Revision()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="T:Galatea.DomainModel.Revision"/> class.
        /// </summary>
        /// <param name="reviewer">A domain model object representing the reviewer.</param>
        /// <param name="comments">The review comments.</param>
        /// <param name="revisedContent">The revised review content.</param>
        /// <exception cref="T:System.ArgumentNullException">
        /// Thrown if any of the values passed to the <paramref name="reviewer"/>,
        /// <paramref name="comments"/> or <paramref name="revisedContent"/> parameters are null
        /// or empty.
        /// </exception>
        public Revision(User reviewer, String comments, String revisedContent)
            : this()
        {
            Contract.Requires(reviewer != null);
            Contract.Requires(String.IsNullOrWhiteSpace(comments) != true);
            Contract.Requires(String.IsNullOrWhiteSpace(revisedContent) != true);

            this.ReviewerGuid     = reviewer.UserGuid;
            this.Comment          = comments;
            this.RevisedContent   = revisedContent;
            this.DateSubmittedUtc = DateTimeOffset.UtcNow;
        }

        /// <summary>
        /// Gets the unique identifier of reviewer user.
        /// </summary>
        public Guid ReviewerGuid { get; internal set; }

        private String _comment;

        /// <summary>
        /// Gets or sets the comments.
        /// </summary>
        /// <remarks>
        /// Setting this property will raise the <see cref="E:System.ComponentModel.INotifyPropertyChanged.PropertyChanged"/>
        /// event.
        /// </remarks>
        /// <exception cref="T:System.ArgumentNullException">
        /// Thrown if the value passed to the mutator accessor is null.
        /// </exception>
        public String Comment
        {
            get { return this._comment; }
            internal set
            {
                if (value != this.Comment)
                {
                    this._comment = value;

                    this.OnPropertyChanged(new PropertyChangedEventArgs("Comments"));
                }
            }
        }

        private String _revisedContent;

        /// <summary>
        /// Gets or sets the revised content.
        /// </summary>
        /// <remarks>
        /// Setting this property will raise the <see cref="E:System.ComponentModel.INotifyPropertyChanged.PropertyChanged"/>
        /// event.
        /// </remarks>
        /// <exception cref="T:System.ArgumentNullException">
        /// Thrown if the value passed to the mutator accessor is null.
        /// </exception>
        public String RevisedContent
        {
            get { return this._revisedContent; }
            internal set
            {
                if (value != this.RevisedContent)
                {
                    this._revisedContent = value;

                    this.OnPropertyChanged(new PropertyChangedEventArgs("RevisedContent"));
                }
            }
        }

        /// <summary>
        /// Gets the date &amp time on which the review was submitted.
        /// </summary>
        public DateTimeOffset DateSubmittedUtc { get; internal set; }

        /// <summary>
        /// Update the comment.
        /// </summary>
        /// <param name="newComment">The new value of the revision comment.</param>
        public void UpdateComment(String newComment)
        {
            Contract.Requires(String.IsNullOrWhiteSpace(newComment) != true);

            this.Comment = newComment;
        }
        
        /// <summary>
        /// Updates the revised content of the revision.
        /// </summary>
        /// <param name="newContent">The new revised content value.</param>
        public void UpdateRevisedContent(String newContent)
        {
            Contract.Requires(String.IsNullOrWhiteSpace(newContent) != true);

            this.RevisedContent = newContent;
        }
    }
}