﻿namespace Galatea.DomainModel
{
    using System;
    using System.ComponentModel;
    using System.Diagnostics.Contracts;
    using System.Globalization;
    using System.Threading;

    /// <summary>
    /// Defines a domain model object representing a user.
    /// </summary>
    public class User : INotifyPropertyChanged
    {
        /// <summary>
        /// Occurs when a property value changes.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Raises the <see cref="E:System.ComponentModel.INotifyPropertyChanged.PropertyChanged"/> event.
        /// </summary>
        /// <param name="e">The <see cref="System.ComponentModel.PropertyChangedEventArgs"/>
        /// instance containing the event data.</param>
        protected virtual void OnPropertyChanged(PropertyChangedEventArgs e)
        {
            var handler = Interlocked.CompareExchange(ref this.PropertyChanged, null, null);

            if (handler != null)
                handler(this, e);
        }

        /// <summary>
        /// Prevents a default instance of the <see cref="T:Galatea.DomainModel.User"/> class from
        /// being created.
        /// </summary>
        internal User()
        {
            this.Projects = new ProjectRoleMap();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="T:Galatea.DomainModel.User"/> class.
        /// </summary>
        /// <param name="emailAddress">The users email address.</param>
        /// <param name="passwordHash">The hashed representation of the users password.</param>
        /// <param name="culture">The users local culture.</param>
        /// <exception cref="T:System.ArgumentNullException">
        /// Thrown if any of the values passed to the <paramref name="emailAddress"/>,
        /// <paramref name="passwordHash"/> or <paramref name="culture"/> parameters are null or
        /// empty.
        /// </exception>
        public User(String emailAddress, String passwordHash, CultureInfo culture)
            : this()
        {
            Contract.Requires(String.IsNullOrWhiteSpace(emailAddress) != true, "emailAddress");
            Contract.Requires(String.IsNullOrWhiteSpace(passwordHash) != true, "passwordHash");
            Contract.Requires(culture != null, "culture");

            this.EmailAddress  = emailAddress;
            this.PasswordHash  = passwordHash;
            this.Culture       = culture;
            this.LastLoginUtc  = DateTimeOffset.UtcNow;
        }

        /// <summary>
        /// Gets the users GUID.
        /// </summary>
        public Guid UserGuid { get; internal set; }

        private String _emailAddress;

        /// <summary>
        /// Gets or sets the users email address.
        /// </summary>
        /// <remarks>
        /// Setting this property will raise the <see cref="E:System.ComponentModel.INotifyPropertyChanged.PropertyChanged"/>
        /// event.
        /// </remarks>
        /// <exception cref="T:System.ArgumentNullException">
        /// Thrown if the value passed to the mutator accessor is null or empty.
        /// </exception>
        public String EmailAddress
        {
            get { return this._emailAddress; }
            internal set
            {
                Contract.Requires(String.IsNullOrWhiteSpace(value) != true, "value");

                if (value != this.EmailAddress)
                {
                    this._emailAddress = value;

                    this.OnPropertyChanged(new PropertyChangedEventArgs("EmailAddress"));
                }
            }
        }

        private String _passwordHash;

        /// <summary>
        /// Gets or sets a hashed representation of the users password.
        /// </summary>
        /// <remarks>
        /// Setting this property will raise the <see cref="E:System.ComponentModel.INotifyPropertyChanged.PropertyChanged"/>
        /// event.
        /// </remarks>
        /// <exception cref="T:System.ArgumentNullException">
        /// Thrown if the value passed to the mutator accessor is null or empty.
        /// </exception>
        public String PasswordHash
        {
            get { return this._passwordHash; }
            internal set
            {
                Contract.Requires(String.IsNullOrWhiteSpace(value) != true, "value");

                if (value != this.PasswordHash)
                {
                    this._passwordHash = value;

                    this.OnPropertyChanged(new PropertyChangedEventArgs("PasswordHash"));
                }
            }
        }

        private CultureInfo _culture;

        /// <summary>
        /// Gets or sets the users culture.
        /// </summary>
        /// <remarks>
        /// Setting this property will raise the <see cref="E:System.ComponentModel.INotifyPropertyChanged.PropertyChanged"/>
        /// event.
        /// </remarks>
        /// <exception cref="T:System.ArgumentNullException">
        /// Thrown if the value passed to the mutator accessor is null.
        /// </exception>
        public CultureInfo Culture
        {
            get { return this._culture; }
            internal set
            {
                Contract.Requires(value != null, "value");

                if (value != this.Culture)
                {
                    this._culture = value;

                    this.OnPropertyChanged(new PropertyChangedEventArgs("Culture"));
                }
            }
        }

        private DateTimeOffset _lastLoginUtc;

        /// <summary>
        /// Gets or sets the date &amp; time on which the user last logged in.
        /// </summary>
        /// <remarks>
        /// Setting this property will raise the <see cref="E:System.ComponentModel.INotifyPropertyChanged.PropertyChanged"/>
        /// event.
        /// </remarks>
        public DateTimeOffset LastLoginUtc
        {
            get { return this._lastLoginUtc; }
            internal set
            {
                if (value != this.LastLoginUtc)
                {
                    this._lastLoginUtc = value;

                    this.OnPropertyChanged(new PropertyChangedEventArgs("LastLoginUtc"));
                }
            }
        }

        /// <summary>
        /// Gets or sets the projects the user is involved with, and their role within the project.
        /// </summary>
        public ProjectRoleMap Projects { get; internal set; }
    }
}