﻿namespace Galatea.Repositories
{
    using System;
    using System.Diagnostics.Contracts;
    using AutoMapper;
    using Galatea.Data;
    using Galatea.Data.TransportObjects;
    using Galatea.DomainModel;

    /// <summary>
    /// Acts as a mediator betwen the domain layer and the data access layer for User entities.
    /// </summary>
    public class UserRepository
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="UserRepository"/> class.
        /// </summary>
        /// <param name="dataGatewayFactory">The data gateway factory.</param>
        public UserRepository(IGalateaDataGatewayFactory dataGatewayFactory)
        {
            Contract.Requires(dataGatewayFactory != null, "dataGatewayFactory");

            this.DataGatewayFactory = dataGatewayFactory;
        }

        /// <summary>
        /// Gets or sets the data gateway factory.
        /// </summary>
        private IGalateaDataGatewayFactory DataGatewayFactory { get; set; }

        /// <summary>
        /// Creates a new user.
        /// </summary>
        /// <param name="value">A domain model object representing the user to create.</param>
        /// <returns>A new domain model object representing the new user.</returns>
        public void Create(ref User value)
        {
            Contract.Requires(value != null);

            // Get a data gateway implementation from the data gateway factory...
            using (var dataGateway = this.DataGatewayFactory.GetUserDataGateway())
            {
                // Create a new DTO by mapping from the domain model object...
                var dto = Mapper.Map<UserDTO>(value);

                // Pass the DTO into the datagateway for persistance, and receive a new DTO back
                // which represents the persisted entity...
                dataGateway.Create(ref dto);

                // Map the returned DTO back into a domain model object...
                value = Mapper.Map<User>(dto);
            }
        }

        public User GetUser(Guid userGuid)
        {
            // Get an IUserDataGateway from the data gateway factory...
            using (var dataGateway = this.DataGatewayFactory.GetUserDataGateway())
            {
                // Get a data transport object from the data gateway...
                var dto = dataGateway.GetUser(userGuid);

                // Using the data mapper, convert the DTO into a domain model object...
                return Mapper.Map<User>(dto);
            }
        }
    }
}